-- Created by NuclearTonic
-- If reusing this code, please keep the name of the original author listed
-- in respect for borrowing said authors code.
-- RESPECT!

import "Turbine";
import "Turbine.UI";
import "Tonic.Utils.Class";
import "Tonic.TonicBars.Bars.BaseBar";
import "Tonic.TonicBars.Bars.Tabbed.QuickslotBar";
import "Tonic.TonicBars.Bars.Extension.ExtensionBar";
import "Tonic.TonicBars.Bars.Window.WindowBar";

BarManager = class( Turbine.Object );

function BarManager:Constructor()
	self.working = false;
	self.RegisteredBars = { };
	
	self:Construct( theSettingsManager:GetBars() );
end

function BarManager:RefreshBars()
	for key, value in pairs (self.RegisteredBars) do
		value:Refresh();
	end
end

function BarManager:Add( barType, cBarID, cQuickslotID )
	if ( self.working == false ) then
		self.working = true;
		local settings = theSettingsManager:GetSettings();
		local added = settings.nextBarId;
		Turbine.Shell.WriteLine( "Adding Bar: " .. settings.nextBarId	);
		if ( barType == QUICKSLOTBAR ) then
			tonic = Tonic.TonicBars.Bars.Tabbed.QuickslotBar( settings.nextBarId );
		elseif ( barType == EXTENSIONBAR ) then
			local barSettings = theSettingsManager:GetBarSettings( cBarID );
			if ( barSettings.barType == QUICKSLOTBAR ) then
				tonic = Tonic.TonicBars.Bars.Extension.ExtensionBar( settings.nextBarId );
				self.RegisteredBars[cBarID]:RegisterBarExtension( tonic, cQuickslotID, settings.nextBarId );
			else
				Turbine.Shell.WriteLine( "You can not add an extension to an extension." );
			end
		elseif ( barType == WINDOWBAR ) then
			tonic = Tonic.TonicBars.Bars.Window.WindowBar( settings.nextBarId );
		end
		if ( tonic ~= nil ) then
			self.RegisteredBars[settings.nextBarId] = tonic;
			theSettingsManager:IncrementNextId();
			local barSettings = theSettingsManager:GetBarSettings( added );
			theSettingsManager:SetBarSettings( added, barSettings );
			self.working = false;
			return added;
		else
			self.working = false;
			return nil;
		end
	else
		Turbine.Shell.WriteLine( "Your actions are occuring too quickly, please slow down." );
		return nil;
	end
end

function BarManager:Remove( barid )
	if ( barid ~= nil ) then
		Turbine.Shell.WriteLine( "Bar Being Removed:" .. barid );
		eventManager:UnregisterForEvents( barid );

		self.RegisteredBars[barid]:SetVisible( false );
		self.RegisteredBars[barid]:SetParent( nil );
		self.RegisteredBars[barid]:ClearQuickslots( true );
		self.RegisteredBars[barid]:SetVisible( false );
		self.RegisteredBars[barid]:SetBackColor( Turbine.UI.Color( 0, 0, 0, 0) );
		self.RegisteredBars[barid] = nil;
		
		theSettingsManager:SetBarSettings( barid, nil);
	end
end

function BarManager:Copy( barid )
	if ( barid ~= nil and self.working == false ) then
		self.working = true;
		local barsettings = theSettingsManager:GetBarSettings( barid );
		copyBarSettings = {};
		self.working = false;
		local added = self:Add();
		self.working = true;
		self.RegisteredBars[added]:SetVisible( true );
		theSettingsManager:deepcopy( barsettings, copyBarSettings );
		theSettingsManager:SetBarSettings( added, copyBarSettings );
		self.working = false;
		return added;
	else
		return nil;
	end
end

function BarManager:Reset( barid )
	if ( barid ~= nil and self.working == false ) then
		self.working = true;	
		theSettingsManager:SetBarSettings( barid, nil, false );
		local barsettings = theSettingsManager:GetBarSettings( barid );
		theSettingsManager:SetBarSettings( barid, barsettings );
		self.working = false;
	else
		return nil;
	end
end

function BarManager:Construct( storedBars )
	for key, value in pairs (storedBars) do
		if ( value.barType == QUICKSLOTBAR ) then
			tonic = Tonic.TonicBars.Bars.Tabbed.QuickslotBar( tonumber(key) );
			self.RegisteredBars[tonumber(key)] = tonic;
		end
	end

	for key, value in pairs (storedBars) do
		if ( value.barType == EXTENSIONBAR ) then
			tonic = Tonic.TonicBars.Bars.Extension.ExtensionBar( key );
			local attachedbarSettings = theSettingsManager:GetBarSettings( key );
			local thebar = self.RegisteredBars[attachedbarSettings.connectionBarID];
			if ( thebar ~= nil ) then
				thebar:RegisterBarExtension( tonic, attachedbarSettings.connectionQuickslotID, key );
				self.RegisteredBars[tonumber(key)] = tonic;
			else
				theSettingsManager:SetBarSettings( key, nil);
				Turbine.Shell.WriteLine( "We found extension bar without a main bar. Deleting. Thank the community for the fix." );
			end
		end
	end
	
	for key, value in pairs (storedBars) do
		--RegisteredBars[tonumber(key)]:UpdateBarExtensions();
	end
end

function BarManager:GetBars()
	return self.RegisteredBars;
end

function BarManager:ShowExtensionBarMenu( barid )
	if ( self.RegisteredBars[ barid ].barType == QUICKSLOTBAR ) then
		self.RegisteredBars[ barid ]:ShowBarMenu();
	end
end

function BarManager:UpdateBarExtensions()
	for key, value in pairs (self.RegisteredBars) do
		local barsettings = theSettingsManager:GetBarSettings( key );
		if ( barsettings.barType == QUICKSLOTBAR and value ~= nil ) then
			self.RegisteredBars[ key ]:UpdateBarExtensions();
		end
	end
end

function BarManager:Alive( barid )
	if ( self.RegisteredBars[barid] == nil ) then
		return false;
	else
		return true;
	end
end

function BarManager:ExtensionMousedOver()
end

function BarManager:ExtensionMousedLeft()
end