-- Created by NuclearTonic
-- The use of this code requires the permission of the author.
-- Permission can be atained by contacting NuclearTonic at: TonicBars@gmail.com
--
-- RESPECT!

import "Tonic.TonicBars.Bars.QuickslotList";

BaseBar = class( Turbine.UI.Window );

function BaseBar:Constructor()
	Turbine.UI.Window.Constructor( self );

	local bSettings = theSettingsManager:GetBarSettings( self.id );
	self.dragged = false;
	self.qsCreated = false;
	self.isSelectedOnMainMenu = false;
	self.f12HideBar = false;
	
	self:SetMouseVisible( true );
	self:SetAllowDrop( true );
	self:SetPosition( bSettings.x, bSettings.y );

	self:Create();

	local bSettings = theSettingsManager:GetBarSettings( self.id );
	self.quickslotList.loading = true;
	theSettingsManager:LoadQuickslots( bSettings, self.quickslotList.quickslots );
	self.quickslotList.loading = false;
end

function BaseBar:Create()
	local barSettings = theSettingsManager:GetBarSettings( self.id );
	
	self.quickslotList = QuickslotList( self.id );
	self.quickslotList:SetParent( self );
	self.quickslotList:SetMaxItemsPerLine( barSettings.quickslotColumns );
	self.quickslotList:SetPosition(0, 0 );

	self.qsCreated = true;

	self:Refresh();
end

function BaseBar:ClearQuickslots()
	self.quickslotList:ClearQuickslots()
end

function BaseBar:Refresh()
	local settings = theSettingsManager:GetSettings();
	local barSettings = theSettingsManager:GetBarSettings( self.id );
	if ( settings.barMode ~= NORMAL_MODE ) then
		if (self.id == settings.selectedBar) then
			self:SetOpacity( 0.5 );
			self:SetBackColor( Turbine.UI.Color(1,0,1,0) );
		else
			if ( (barSettings.barType == QUICKSLOTBAR and settings.barMode == QUICKSLOT_MODE) or (barSettings.barType == EXTENSIONBAR and settings.barMode == EXTENSION_MODE) ) then
				self:SetOpacity( 0.25 );
				self:SetBackColor( Turbine.UI.Color(1,0,1,0) );
			else
				if ( barSettings.barType == QUICKSLOTBAR ) then
					self:SetOpacity( 0.5 );
					self:SetBackColor( Turbine.UI.Color(1,0.4,0.6,0.8) );
				else
					self:SetOpacity( 0.15 );
				end
			end
		end

		if ( barSettings.barType == EXTENSIONBAR and settings.barMode ~= NORMAL_MODE ) then
			self:SetBackColor( Turbine.UI.Color(1,1,1,0) );
		end
		self:SetVisible( true );
	else
		if ( self.faded ) then
			self:SetOpacity( barSettings.fadeOpacity );
		else
			self:SetOpacity( 1 );
		end
		if ( barSettings.useBackgroundColor == true ) then
			local tempColor = Turbine.UI.Color( barSettings.opacity, barSettings.backgroundColorRed, barSettings.backgroundColorGreen, barSettings.backgroundColorBlue);
			self:SetBGColor( tempColor, true );
		else
			self:SetBGColor( Turbine.UI.Color( 0, 0, 0, 0), false );
		end
		if ( barSettings.barType == QUICKSLOTBAR ) then
			self:RefreshKeys( false );
		elseif ( barSettings.barType == EXTENSIONBAR ) then
			self:SetVisible( false );
		end
	end
	
	self.quickslotList:SetAllowDrop( not barSettings.locked );
	self.quickslotList:SetMaxItemsPerLine( barSettings.quickslotColumns );
	local showQuickslots = (settings.barMode ~= NORMAL_MODE) or barSettings.locked == false;
	local lockQuickslots = barSettings.locked and settings.barMode == NORMAL_MODE;
	self.quickslotList:Refresh( showQuickslots, lockQuickslots );

	self:SetBarSize();
end

function BaseBar:SetBarSize()
	self:SetSize( self.quickslotList:GetWidth(), self.quickslotList:GetHeight() );
end

function BaseBar:DetermineVisiblity()
	if ( not self.f12HideBar ) then
		self:SetVisible( true );
	else
		self:SetVisible( false );
	end
end

function BaseBar:F12Pressed()
	self.f12HideBar = not self.f12HideBar;
	self:DetermineVisiblity();
end

function BaseBar:GetQuickslotList()
	return self.quickslotList;
end

function BaseBar:GetID()
	return self.id;
end