-- Created by NuclearTonic
-- The use of this code requires the permission of the author.
-- Permission can be atained by contacting NuclearTonic at: TonicBars@gmail.com
--
-- RESPECT!

import "Tonic.Utils.Class";
import "Tonic.UI.Window";

WindowBar = class( Tonic.TonicBars.Bars.BaseBar );

function WindowBar:Constructor( barid )
	self.id = barid;
	self.mainWindow = Tonic.UI.Window();
	
	self.watchSizeChanges = false;
	self.changingSizes = false;
	
	local thebarSettings = theSettingsManager:GetBarSettings( self.id );
	if ( thebarSettings.barType ~= WINDOWBAR ) then
		thebarSettings.barType = WINDOWBAR;
		thebarSettings.quickslotRows = 2;
		thebarSettings.quickslotColumns = 4;
		thebarSettings.quickslotCount = 8;
		thebarSettings.visible = true;
		theSettingsManager:SetBarSettings( self.id, thebarSettings );
	end

	Tonic.TonicBars.Bars.BaseBar.Constructor( self );

	self:SetParent( self.mainWindow );
	self:SetPosition( 10, 41 );
	self.isVisible = true;

	self.mainWindow.SizeChanged = function(sender,args)
		if ( self.changingSizes == false ) then
			self.changingSizes = true;
			local barSettings = theSettingsManager:GetBarSettings( self.id );
			local originalCols = barSettings.quickslotColumns;
			local originalRows = barSettings.quickslotRows;

			barSettings.quickslotColumns = math.floor(((sender:GetWidth() - 24) / barSettings.quickslotSize) * 1.1);
			barSettings.quickslotRows = math.floor(((sender:GetHeight() - 55) / barSettings.quickslotSize) * 1.1);
			barSettings.quickslotCount = barSettings.quickslotRows * barSettings.quickslotColumns;
			if ( self.watchSizeChanges and (originalCols ~= barSettings.quickslotColumns or originalRows ~= barSettings.quickslotRows) ) then
				self.quickslotList:SetMaxItemsPerLine( barSettings.quickslotColumns );
				theSettingsManager:SetBarSettings( self.id, barSettings );
				self:SetSize( self.quickslotList:GetWidth(), self.quickslotList:GetHeight() );
			end
			self.changingSizes = false;
		end
	end
	self.mainWindow.MouseEnter = function(sender,args)
		local barSettings = theSettingsManager:GetBarSettings( self.id );
		if ( maitre_d:Alive( self.id ) and barSettings.useFading == true ) then
			self.faded = false;
			self:Refresh();
		end
	end
	self.mainWindow.MouseLeave = function(sender,args)
		local barSettings = theSettingsManager:GetBarSettings( self.id );
		if ( maitre_d:Alive( self.id ) and barSettings.useFading == true ) then
			self.faded = true;
			self:Refresh();
		end
	end
	self.mainWindow.PositionChanged = function( sender, args )
		local settings = theSettingsManager:GetSettings();
		local barSettings = theSettingsManager:GetBarSettings( self.id );
		if ( maitre_d:Alive( self.id ) and settings.barMode ~= NORMAL_MODE ) then
			local x,y = self.mainWindow:GetPosition();
			barSettings.x = x;
			barSettings.y = y;
			theSettingsManager:SetBarSettings( self.id, barSettings );
			maitre_d:UpdateBarExtensions();
		end
	end
	self.mainWindow.MouseDown = function( sender, args )
		if ( maitre_d:Alive( self.id ) == true ) then
			local settings = theSettingsManager:GetSettings();
			if ( settings.barMode ~= NORMAL_MODE ) then
				if ( args.Button == Turbine.UI.MouseButton.Left ) then
					self.dragStartX = args.X;
					self.dragStartY = args.Y;
					self.dragging = true;
					self.dragged = false;
				end
			end
		end
	end
	self.mainWindow.MouseMove = function( sender, args )
		local left, top = self:GetPosition();
		local settings = theSettingsManager:GetSettings();
		if ( maitre_d:Alive( self.id ) and settings.barMode ~= NORMAL_MODE and self.dragging ) then
			self.mainWindow:SetPosition( left + ( args.X - self.dragStartX ), top + ( args.Y - self.dragStartY ) );
			self.dragged = true;
			maitre_d:UpdateBarExtensions();
		end
	end
	self.mainWindow.MouseUp = function( sender, args )
		if ( maitre_d:Alive( self.id ) == true and args.Button == Turbine.UI.MouseButton.Left ) then
			self.dragging = false;
		  
			if( self.dragged ) then
				local x, y = self.mainWindow:GetPosition();
				local validX = x;
				local validY = y;
				if( x < 0 ) then
					x = 0;
				end
		
				if( y < 0 ) then
					y = 0;
				end
				if( x + self.mainWindow:GetWidth() > Turbine.UI.Display.GetWidth() ) then
					x = Turbine.UI.Display.GetWidth() - self.mainWindow:GetWidth();
				end
				if( y + self.mainWindow:GetHeight() > Turbine.UI.Display.GetHeight() ) then
					y = Turbine.UI.Display.GetHeight() - self.mainWindow:GetHeight();
				end
		
				self.mainWindow:SetPosition( x, y );
			end
		end
	end
	self.mainWindow.KeyDown = function( sender, args )
		if ( maitre_d:Alive( self.id ) == true ) then
			local changed = false;
			if ( args.Action == 268435635 ) then	-- F12
				self.isVisible = not self.isVisible;
				self:SetVisible( self.isVisible );
				changed = true;
			end
			
			if ( changed ) then
				self:RefreshKeys( self.altIsDown, self.controlIsDown, self.shiftIsDown );
			end
		end
	end
	eventManager:RegisterForEvents( self, self.id );
end

function WindowBar:Create()
	local barSettings = theSettingsManager:GetBarSettings( self.id );
	
	self.quickslotList = QuickslotList( self.id );
	self.quickslotList:SetParent( self );

	self.qsCreated = true;
	self:Refresh();

	self:SetSize( self.quickslotList:GetWidth(), self.quickslotList:GetHeight() );
	self.mainWindow:SetSize( self.quickslotList:GetWidth() + 24, self.quickslotList:GetHeight() + 55 );
	self.quickslotList:SetPosition( 0, 0 );
	self.watchSizeChanges = true;
end

function WindowBar:SetBarSize(xsize, ysize)
	if ( xsize ~= nil or ysize ~= nil ) then
		self.mainWindow:SetSize( xsize, ysize );
	else
		self.mainWindow:SetSize( self.quickslotList:GetWidth() + 24, self.quickslotList:GetHeight() + 55);
	end
end

function WindowBar:SetBGColor( color )
	self:SetBackColor( color );
end

function WindowBar:Refresh()
	Tonic.TonicBars.Bars.BaseBar.Refresh( self );
	eventManager:NotifyClients();
end

-- This function should ONLY be called by the Event Manager. Doing so in any other copacity
-- can and will cause issues with the visibility of the bars.
--
-- It is recommended to call: "eventManager:NotifyClients();" if needed.
function WindowBar:DetermineVisiblity( eventValue )
	local settings = theSettingsManager:GetSettings();
	local barSettings = theSettingsManager:GetBarSettings( self.id );

	if ( settings.barMode == NORMAL_MODE ) then
		local visible = false;
		if ( eventValue ~= nil and eventValue ) then
			visible = true;
		end
		if ( visible == false and self.isVisible == true ) then
			visible = barSettings.visible;
		end
	
		if ( self.f12HideBar ) then
			visible = false;
		end
		self.mainWindow:SetVisible( visible );
		self:SetVisible( visible );
	end
end

function WindowBar:ClearQuickslots()
	self.quickslotList:ClearQuickslots();
end