-- Created by NuclearTonic
-- The use of this code requires the permission of the author.
-- Permission can be atained by contacting NuclearTonic at: TonicBars@gmail.com
--
-- RESPECT!

import "Turbine.Gameplay";
import "Turbine.UI";
import "Tonic.Utils.Class";

EventManager = class( Turbine.Object  );

function EventManager:Constructor()
	self.effectsRegistered = false;
	
	if ( self.registeredEvents == nil ) then
		self.registeredEvents = { };
	end
	if ( self.registeredEvents.classes == nil ) then
		self.registeredEvents.classes = { };
	end
	self.registeredEvents.playerInCombat = false;
	
	self:SetupKeyEvents();
	self:RegisterForCombatCallbacks();
	self:RegisterForHealthCallbacks();
	self:RegisterForPowerCallbacks();	
end

function EventManager:RegisterForEvents( registerant, rid )
	if ( self.clients == nil ) then
		self.clients = { };
	end
	if ( self.clients[rid] == nil ) then
		self.clients[rid] = { };
	end

	self.clients[rid].registered = true;
	self.clients[rid].bar = registerant;

	self:NotifyClients();
end

function EventManager:UnregisterForEvents( rid )
	if ( self.clients ~= nil and self.clients[rid] ~= nil ) then
		self.clients[rid].registered = false;
		self.clients[rid].bar = nil;
		self.clients[rid] = nil;
	end
end

function EventManager:NotifyClients()
	if ( self.clients == nil ) then
		self.clients = { };
	end
	for key, value in pairs (self.clients) do
		local visible = false;
		local barSettings = theSettingsManager:GetBarSettings( key );
		if ( value ~= nil and value.registered == true and barSettings.events ~= nil and maitre_d ~= nil and maitre_d:Alive( key ) == true) then
			-- Send all Effect based events to all clients
			if (barSettings.events.effects ~= nil ) then
				if ( barSettings.events.triggerOnClassBuffActive == true ) then
					for key2, value2 in pairs (self.registeredEvents.classes[ playerClass ].effects) do
						if ( value2 ~= nil and value2.count > 0 and barSettings.events.effects[ key2 ] == true ) then
							visible = true;
						end
					end
				else
					visible = true;
					for key2, value2 in pairs (self.registeredEvents.classes[ playerClass ].effects) do				
						if ( value2 ~= nil and value2.count > 0 and barSettings.events.effects[ key2 ] == true ) then
							visible = false;
						end
					end
				end
			end
			-- Send all Category Effect based events to all clients
			if (barSettings.events.categories ~= nil ) then
				for key3, value3 in pairs (self.registeredEvents.categories) do
					if ( value3 ~= nil and value3.count > 0 and barSettings.events ~= nil and barSettings.events.categories[ key3 ] ~= nil ) then
						visible = true;
					end
				end
			end

			-- Send all Keypress based events to all clients
			if ( (self.registeredEvents.altIsDown ~= nil and barSettings.events.isAlt == true) or 
				 (self.registeredEvents.controlIsDown ~= nil and barSettings.events.isControl == true) or 
				 (self.registeredEvents.shiftIsDown ~= nil and barSettings.events.isShift == true) or
				 (self.registeredEvents.playerInCombat == true and barSettings.events.displayInCombat == true) or
				 (self.registeredEvents.playerInCombat == false and barSettings.events.displayNotInCombat == true)
				) then
				visible = true;
			end
			
			if ( (self.registeredEvents.healthTriggerActive ~= nil and barSettings.events.healthTrigger ~= nil and
			      self.registeredEvents.healthTriggerActive <= barSettings.events.healthTrigger and 
				  barSettings.events.displayOnHealth) or 
				 (self.registeredEvents.powerTriggerActive ~= nil and barSettings.events.powerTrigger ~= nil and
			      self.registeredEvents.powerTriggerActive <= barSettings.events.powerTrigger and 
				  barSettings.events.displayOnPower) ) then
				visible = true;
			end

			-- If 12 is pressed TOGGLE display on all clients
			if ( self.registeredEvents.F12 ~= nil ) then
				value.bar:F12Pressed();
			end

			value.bar:DetermineVisiblity( visible );
		end
	end
end

function EventManager:EffectWatchSetup()
    local effects = player:GetEffects();
	for i = 1, effects:GetCount() do
		local effect = effects:Get(i);
		local name = effect:GetName();
		
		if ( self.registeredEvents.classes[ playerClass ] ~= nil and self.registeredEvents.classes[ playerClass ].effects ~= nil and self.registeredEvents.classes[ playerClass ].effects[name] ~= nil ) then
			self.registeredEvents.classes[ playerClass ].effects[name].count = self.registeredEvents.classes[ playerClass ].effects[name].count + 1;
		end

		local category = effect:GetCategory();
		if ( self.registeredEvents.categories ~= nil and self.registeredEvents.categories[category] ~= nil ) then
			self.registeredEvents.categories[category].count = self.registeredEvents.categories[category].count + 1;
		end
	end

	effects.EffectAdded = function(sender, args)
		local effects = player:GetEffects()
		local effect = effects:Get(args.Index);

		local name = effect:GetName();
		if ( self.registeredEvents.classes[ playerClass ] ~= nil and self.registeredEvents.classes[ playerClass ].effects[name] ~= nil ) then
			self.registeredEvents.classes[ playerClass ].effects[name].count = self.registeredEvents.classes[ playerClass ].effects[name].count + 1;
		end

		local category = effect:GetCategory();
		if ( self.registeredEvents.categories[category] ~= nil ) then
			self.registeredEvents.categories[category].count = self.registeredEvents.categories[category].count + 1;
		end
		self:NotifyClients();
	end
	
	effects.EffectRemoved = function(sender, args)
		local effect = args.Effect;

		local name = effect:GetName();
		if ( self.registeredEvents.classes[ playerClass ] ~= nil and self.registeredEvents.classes[ playerClass ].effects[name] ~= nil ) then
			self.registeredEvents.classes[ playerClass ].effects[name].count = self.registeredEvents.classes[ playerClass ].effects[name].count - 1;
		end

		local category = effect:GetCategory();
		if ( self.registeredEvents.categories[category] ~= nil ) then
			self.registeredEvents.categories[category].count = self.registeredEvents.categories[category].count - 1;
		end
		self:NotifyClients();
	end
	
	effects.EffectsCleared = function(sender, args)
		local effect = args.Effect;

		local name = effect:GetName();
		if ( self.registeredEvents.classes[ playerClass ].effects[name] ~= nil ) then
			self.registeredEvents.classes[ playerClass ].effects[name].count = self.registeredEvents.classes[ playerClass ].effects[name].count - 1;
		end

		local category = effect:GetCategory();
		if ( self.registeredEvents.categories[category] ~= nil ) then
			self.registeredEvents.categories[category].count = self.registeredEvents.categories[category].count - 1;
		end
		self:NotifyClients();
	end
end

function EventManager:SetupKeyEvents()
	-- I hate having to do this... but I need to.
	self.window = Turbine.UI.Window();
	self.window:SetWantsKeyEvents( true );
	self.window.KeyDown = function( sender, args )
		local changed = false;
		if ( args.Action == 268435556 ) then	-- ALT
			self.registeredEvents.altIsDown = true;
			changed = true;
		end
		if ( args.Action == 268435575 ) then	-- CTRL
			self.registeredEvents.controlIsDown = true;
			changed = true;
		end
		if ( args.Action == 268435458 ) then	-- SHIFT
			self.registeredEvents.shiftIsDown = true;
			changed = true;
		end
		if ( args.Action == 268435635 ) then	-- F12
			self.registeredEvents.F12 = true;
			changed = true;
		end

		if ( changed ) then
			self:NotifyClients();
		end
	end

	self.window.KeyUp = function( sender, args )
		local changed = false;
		if ( args.Action == 268435556 and self.registeredEvents.altIsDown ~= nil ) then
			self.registeredEvents.altIsDown = nil;
			changed = true;
		end
		if ( args.Action == 268435575 and self.registeredEvents.controlIsDown ~= nil ) then
			self.registeredEvents.controlIsDown = nil;
			changed = true;
		end
		if ( args.Action == 268435458 and self.registeredEvents.shiftIsDown ~= nil ) then
			self.registeredEvents.shiftIsDown = nil;
			changed = true;
		end
		if ( args.Action == 268435635 and self.registeredEvents.F12 ~= nil ) then
			self.registeredEvents.F12 = nil;
			changed = true;
		end

		if ( changed ) then
			self:NotifyClients();
		end
	end
end

function EventManager:RegisterForCombatCallbacks()
	player.InCombatChanged = function( sender, args )
		if ( player:IsInCombat() ) then
			self.registeredEvents.playerInCombat = true;
		else
			self.registeredEvents.playerInCombat = false;
		end
		self:NotifyClients();
	end
end

function EventManager:RegisterForHealthCallbacks()
	player.MoraleChanged = function( sender, args )
		local value = player:GetMorale() / player:GetMaxMorale();
		if ( value ~= 1 ) then
			self.registeredEvents.healthTriggerActive = value;
		else
			self.registeredEvents.healthTriggerActive = nil;
		end
		self:NotifyClients();
	end
end
	
function EventManager:RegisterForPowerCallbacks()
	player.PowerChanged = function( sender, args )
		local value = player:GetPower() / player:GetMaxPower();
		if ( value ~= 1 ) then
			self.registeredEvents.powerTriggerActive = value;
		else
			self.registeredEvents.powerTriggerActive = nil;
		end
		self:NotifyClients();
	end
end

function EventManager:RegisterEffect( descriptionString, name, class )
	if ( self.registeredEvents.classes[ class ] == nil ) then
		self.registeredEvents.classes[ class ] = { };
	end
	if ( self.registeredEvents.classes[ class ].effects == nil ) then
		self.registeredEvents.classes[ class ].effects = { };
	end
	if ( self.registeredEvents.classes[ class ].effects[name] == nil ) then
		self.registeredEvents.classes[ class ].effects[name] = { };
	end

	self.registeredEvents.classes[ class ].effects[name].description = descriptionString;
	self.registeredEvents.classes[ class ].effects[name].count = 0;
end

function EventManager:RegisterCategory( descriptionString, category )
	if ( self.registeredEvents.categories == nil ) then
		self.registeredEvents.categories = { };
	end
	if ( self.registeredEvents.categories[category] == nil ) then
		self.registeredEvents.categories[category] = { };
	end
	self.registeredEvents.categories[category].description = descriptionString;
	self.registeredEvents.categories[category].count = 0;
end

function EventManager:GetRegisteredEvents()
	return self.registeredEvents;
end