-- Created by NuclearTonic
-- The use of this code requires the permission of the author.
-- Permission can be atained by contacting NuclearTonic at: TonicBars@gmail.com
--
-- RESPECT!

import "Turbine";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Tonic.UI.AutoListBox";
import "Tonic.UI.MenuUtils";

buttonWidth = 85;
selectionWidth = 160;
selectionHeight = 20;
startFromTop = 39;
insideBoxXOffset = selectionWidth + 5;
insideBoxYOffset = startFromTop + 200;

QuickslotsMenu = class( Tonic.UI.AutoListBox );

function QuickslotsMenu:Constructor()
	Tonic.UI.AutoListBox.Constructor( self );
	
	self.utils = Tonic.UI.MenuUtils();
	
	-- Quickslots Menu
	self.utils:AddCategoryBox(self, LOCALESTRINGS.QuickslotsMenu["Quick Bars"]);
	
	self.upperBar = self.utils:AddAutoListBox( self, Turbine.UI.Orientation.Vertical, 0, 0, 0, 0 );
	
	self.utils:AddLabelBox( self, LOCALESTRINGS.QuickslotsMenu["Editing Settings for Bar:"], selectionWidth + 100, selectionHeight );
	
	self.barList = Tonic.UI.ComboBox();
	self.barList:SetSize( 200, 20 );
	self.barList:SetParent( self );
    self.barList.SelectedIndexChanged = function(sender, args)
		local settings = theSettingsManager:GetSettings();
		settings.selectedBar = tonumber(args.selection);
		self.settingsBox:SetVisible( true );
		menu:SetupMainMenuDisplay( QUICKSLOT_MODE );
		maitre_d:RefreshBars();
	end
	self.upperBar:AddItem( self.barList );
	
	self:AddSettings();
	self:AddItem( self.upperBar );

	self:EnableTriggers( false );

	self:AddCommandsBar();
	
	self.sidebar = Turbine.UI.Lotro.ScrollBar();
	self.sidebar:SetOrientation( Turbine.UI.Orientation.Vertical );
	self.sidebar:SetParent( self );
	self.sidebar:SetSize( 8, 380 );
	self.sidebar:SetPosition( windowWidth - selectionWidth - 5, 90 );
	self.sidebar:SetZOrder(60);
	self.settingsBox:SetVerticalScrollBar( self.sidebar );
	
	self.settingsBox:SetSize( windowWidth - selectionWidth + 5, 420 );
	self:SetSize( windowWidth - selectionWidth + 25, 420 );

	self:AddItem( self.settingsBox );	
	self.settingsBox:SetVisible( false );
end

function QuickslotsMenu:AddSettings()
	self.settingsBox = self.utils:AddAutoListBox( self, Turbine.UI.Orientation.Vertical, 0, 0, 0, 0 );

	self.utils:AddLabelBox( self.settingsBox, "", 50, selectionHeight );

	local box0 = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Horizontal, 0, 0, 0, 0 );

	self.utils:AddLabelBox( box0, LOCALESTRINGS.QuickslotsMenu["Name:"], 80, selectionHeight );
	self.barName = self.utils:AddTextBox(box0, "", selectionWidth, selectionHeight + 10 );

	local nameButton = Turbine.UI.Lotro.Button();
	nameButton:SetText( LOCALESTRINGS.QuickslotsMenu["Set"] );
	nameButton:SetSize( buttonWidth, selectionHeight );
	nameButton.MouseClick = function( sender, args )
		local selection = self.barList:GetSelection();
		local barSettings = theSettingsManager:GetBarSettings( selection );
		barSettings.barName = self.barName:GetText();
		self:RefreshComboBox( self.barList:GetSelection() );
		theSettingsManager:SetBarSettings( selection, barSettings );
	end
	box0:AddItem( nameButton );
	
	self.utils:AddCategoryBox(self.settingsBox, LOCALESTRINGS.QuickslotsMenu["General"]);

	self.settingsBox:AddItem( box0 );
	self.barLockedCheckBox = self.utils:AddCheckBox( self.settingsBox, LOCALESTRINGS.QuickslotsMenu["Locked"], selectionWidth, selectionHeight );
	
	self.utils:AddLabelBox( self.settingsBox, LOCALESTRINGS.QuickslotsMenu["Bar Visibility?"], 120, selectionHeight );
	self.visibilityList = Tonic.UI.ComboBox();
	self.visibilityList:SetSize( 200, 20 );
	self.visibilityList:SetParent( self.settingsBox );
	self.visibilityList:AddItem( LOCALESTRINGS.QuickslotVisibility["Always"], 1 );
	self.visibilityList:AddItem( LOCALESTRINGS.QuickslotVisibility["Triggered"], 2 );
	self.settingsBox:AddItem( self.visibilityList );
	
	self.utils:AddCategoryBox(self.settingsBox, LOCALESTRINGS.QuickslotsMenu["Triggers"]);
	local box9 = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Vertical, 0, 0, 0, 0 );
	local box9a = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Horizontal, 0, 0, 0, 0 );
	local box9b = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Horizontal, 0, 0, 0, 0 );
	local box9c = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Horizontal, 0, 0, 0, 0 );
	
	self.combatBarCheckBox  = self.utils:AddCheckBox( box9a, LOCALESTRINGS.QuickslotsDisplay["In Combat"], selectionWidth, selectionHeight );
	self.outOfCombatBarCheckBox  = self.utils:AddCheckBox( box9a, LOCALESTRINGS.QuickslotsDisplay["Out of Combat"], selectionWidth, selectionHeight );
	self.altBarCheckBox     = self.utils:AddCheckBox( box9b, LOCALESTRINGS.QuickslotsDisplay["Alt Pressed"], selectionWidth, selectionHeight );
	self.controlBarCheckBox = self.utils:AddCheckBox( box9b, LOCALESTRINGS.QuickslotsDisplay["Ctrl Pressed"], selectionWidth, selectionHeight );
	self.shiftBarCheckBox   = self.utils:AddCheckBox( box9c, LOCALESTRINGS.QuickslotsDisplay["Shift Pressed"], selectionWidth, selectionHeight );
	
	box9:AddItem( box9a );
	box9:AddItem( box9b );
	box9:AddItem( box9c );

	self.settingsBox:AddItem( box9 );
	local box10 = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Vertical, 0, 0, 0, 0 );
	local events = eventManager:GetRegisteredEvents();
	self.eventCheckboxes = { };
	local count = 0;
	local box = nil;
	for key, value in pairs (events.categories) do
		if ( count % 2 == 0 ) then
			box = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Horizontal, 0, 0, 0, 0 );
		end
		self.eventCheckboxes[key] = self.utils:AddCheckBox( box, value.description, selectionWidth, selectionHeight );
		self.eventCheckboxes[key].CheckedCallback = function( sender, args )
			local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
			if ( self.eventCheckboxes[key].IsChecked ) then
				if ( barSettings.events == nil ) then
					barSettings.events = { }; 
				end
				if ( barSettings.events.categories == nil ) then
					barSettings.events.categories = { };
				end
				barSettings.events.categories[key] = true;
			elseif ( barSettings.events ~= nil and barSettings.events.categories ~= nil and barSettings.events.categories[key] ~= nil) then
				barSettings.events.categories[key] = nil;
			end
			theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
		end
		if ( count % 2 == 1 ) then
			box10:AddItem( box );
		end
		count = count + 1;
	end

	self.settingsBox:AddItem( box10 );
	
	local box12 = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Vertical, 0, 0, 0, 0 );
	
	local box6 = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Horizontal, 0, 0, 0, 0 );
	self.healthTriggerCheckBox = self.utils:AddCheckBox( box12, LOCALESTRINGS.QuickslotsMenu["Health Drops Below this %:"], selectionWidth + 100, selectionHeight );
	self.healthSB = self.utils:AddScrollBar( box6, 0, 0, 100, 200, selectionHeight + 20 );
	box12:AddItem( box6 );
	
	local box7 = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Horizontal, 0, 0, 0, 0 );
	self.powerTriggerCheckBox = self.utils:AddCheckBox( box12, LOCALESTRINGS.QuickslotsMenu["Power Drops Below this %:"], selectionWidth + 100, selectionHeight );
	self.powerSB = self.utils:AddScrollBar( box7, 0, 0, 100, 200, selectionHeight + 20 );
	box12:AddItem( box7 );
	
	self.settingsBox:AddItem( box12 );
	
	count = 0;
	box = nil;
	self.utils:AddCategoryBox(self.settingsBox, LOCALESTRINGS.QuickslotsMenu["Class Triggers"]);
	
	self.utils:AddLabelBox( self.settingsBox, LOCALESTRINGS.ClassTriggerType["Triggered When:"], 120, selectionHeight );
	self.triggerList = Tonic.UI.ComboBox();
	self.triggerList:SetSize( 200, 20 );
	self.triggerList:SetParent( self.settingsBox );
	self.triggerList:AddItem( LOCALESTRINGS.ClassTriggerType["Buffs Active"], 1 );
	self.triggerList:AddItem( LOCALESTRINGS.ClassTriggerType["Buffs Not Active"], 2 );
	self.settingsBox:AddItem( self.triggerList );
	
	self.fadeOpacityNum1 = self.utils:AddLabelBox( self.settingsBox, "", 50, selectionHeight );
	
	local box11 = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Vertical, 0, 0, 0, 0 );
	if ( events.classes ~= nil and events.classes[playerClass] ~= nil and events.classes[playerClass].effects ~= nil ) then
		for key, value in pairs (events.classes[playerClass].effects) do
			if ( count % 2 == 0 ) then
				box = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Horizontal, 0, 0, 0, 0 );
			end
			self.eventCheckboxes[key] = self.utils:AddCheckBox( box, value.description, selectionWidth + 15, selectionHeight );
			self.eventCheckboxes[key].CheckedCallback = function( sender, args )
				local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
				if ( self.eventCheckboxes[key].IsChecked ) then
					if ( barSettings.events == nil ) then
						barSettings.events = { }; 
					end
					if ( barSettings.events.effects == nil ) then
						barSettings.events.effects = { };
					end
					barSettings.events.effects[key] = true;
				elseif ( barSettings.events ~= nil and barSettings.events.effects ~= nil and barSettings.events.effects[key] ~= nil) then
					barSettings.events.effects[key] = nil;
				end
				theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
			end
			if ( count % 2 == 1 ) then
				box11:AddItem( box );
			end
			count = count + 1;
		end
	end
	-- because of the count = count + 1;
	if ( count % 2 == 1 ) then
		box11:AddItem( box );
	end	
	self.settingsBox:AddItem( box11 );
	
	self.utils:AddCategoryBox(self.settingsBox, LOCALESTRINGS.QuickslotsMenu["Quick Slots"]);
	
	local box2a = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Vertical );
	local box2b = self.utils:AddAutoListBox( self.box2a, Turbine.UI.Orientation.Horizontal );

	self.label1 = self.utils:AddLabelBox( box2a, LOCALESTRINGS.QuickslotsMenu["Rows:"], selectionWidth + 100, selectionHeight + 5 );
	self.sb = self.utils:AddScrollBar( box2b, 1, 1, 50, 200, selectionHeight + 20 );

	box2a:AddItem( box2b );
	self.settingsBox:AddItem( box2a );
	local box3a = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Vertical );
	local box3b = self.utils:AddAutoListBox( self.box3a, Turbine.UI.Orientation.Horizontal );
	
	self.label2 = self.utils:AddLabelBox( box3a, LOCALESTRINGS.QuickslotsMenu["Columns:"], selectionWidth + 100, selectionHeight + 5 );
	self.sb2 = self.utils:AddScrollBar( box3b, 1, 1, 50, 200, selectionHeight + 20 );

	box3a:AddItem( box3b );
	self.settingsBox:AddItem( box3a );
	local box5a = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Vertical );
	local box5b = self.utils:AddAutoListBox( self.box5a, Turbine.UI.Orientation.Horizontal );
	
	self.label3 = self.utils:AddLabelBox( box5a, LOCALESTRINGS.QuickslotsMenu["Spacing:"], selectionWidth + 100, selectionHeight + 5 );
	self.spacingSB = self.utils:AddScrollBar( box5b, 1, 0, 50, 200, selectionHeight + 20 );
	
	box5a:AddItem( box5b );
	self.settingsBox:AddItem( box5a );
	local box6a = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Vertical );
	local box6b = self.utils:AddAutoListBox( self.box6a, Turbine.UI.Orientation.Horizontal );

	self.label4 = self.utils:AddLabelBox( box6a, LOCALESTRINGS.QuickslotsMenu["Size:"], selectionWidth + 50, selectionHeight + 5 );
	self.sizeSB = self.utils:AddScrollBar( box6b, 16, 0, 20, 200, selectionHeight + 20 );
	
	box6a:AddItem( box6b );
	self.settingsBox:AddItem( box6a );

	self.utils:AddCategoryBox(self.settingsBox, LOCALESTRINGS.QuickslotsMenu["Background Colors"]);
	
	local box10 = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Vertical, 0, 0, 0, 0 );
	
	local box8a = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Vertical, 0, 0, 0, 0 );

	self.useBackgroundCheckBox  = self.utils:AddCheckBox( box8a, LOCALESTRINGS.QuickslotsMenu["Use Background Color on Bar"], selectionWidth + 150, selectionHeight );

	local lb = self.utils:AddLabelBox( box8a, LOCALESTRINGS.QuickslotsMenu["Red:"], 50, selectionHeight );
	self.bgRSB = self.utils:AddScrollBar( box8a, 0, 0, 100, 200, selectionHeight + 20 );
	lb = self.utils:AddLabelBox( box8a, LOCALESTRINGS.QuickslotsMenu["Green:"], 50, selectionHeight );
	self.bgGSB = self.utils:AddScrollBar( box8a, 0, 0, 100, 200, selectionHeight + 20 );
	lb = self.utils:AddLabelBox( box8a, LOCALESTRINGS.QuickslotsMenu["Blue:"], 50, selectionHeight );
	self.bgBSB = self.utils:AddScrollBar( box8a, 0, 0, 100, 200, selectionHeight + 20 );
	lb = self.utils:AddLabelBox( box8a, LOCALESTRINGS.QuickslotsMenu["Opasity:"], 150, selectionHeight );
	self.bgASB = self.utils:AddScrollBar( box8a, 0, 0, 100, 200, selectionHeight + 20 );
	
	box10:AddItem( box8a );
	self.settingsBox:AddItem( box10 );

	self.utils:AddCategoryBox(self.settingsBox, LOCALESTRINGS.QuickslotsMenu["Fade Options"]);

	local box4a = self.utils:AddAutoListBox( self.settingsBox, Turbine.UI.Orientation.Vertical, 0, 0, 0, 0 );
	lb = self.utils:AddLabelBox( box4a, LOCALESTRINGS.QuickslotsMenu["Shortcut Opacity:"], 200, selectionHeight );
	self.fadeOpacitySelection = self.utils:AddScrollBar( box4a, 0, 0, 100, 200, selectionHeight + 20 );
	self.useFadingCheckBox  = self.utils:AddCheckBox( box4a, LOCALESTRINGS.QuickslotsMenu["Remove opacity when moused over?"], selectionWidth + 200, selectionHeight );

	self.fadeOpacityNum1 = self.utils:AddLabelBox( box4a, "", 50, selectionHeight );
	self.fadeOpacityNum3 = self.utils:AddLabelBox( box4a, "", 50, selectionHeight );
	self.settingsBox:AddItem( box4a );
end

function QuickslotsMenu:DisplaySettings( showQuickslots )
	self.settingsBox:SetVisible( showQuickslots );
	
	local localBarSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
	if ( localBarSettings.events == nil ) then
		localBarSettings.events = { };
	end

	self.barName.FocusLost = function(sender, args)
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		barSettings.barName = self.barName:GetText();
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );		
	end
	self.barName:SetText( localBarSettings.barName );
	
    self.visibilityList.SelectedIndexChanged = function(sender, args)
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		if ( self.visibilityList:GetSelection() == 1 ) then
			barSettings.visible = true;
		else
			barSettings.visible = false;
		end
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
		
		self:EnableTriggers( not barSettings.visible );
	end
	if ( localBarSettings.visible ) then
		self.visibilityList:SetSelection( 1 );
	else
		self.visibilityList:SetSelection( 2 );
	end
	
	self.barLockedCheckBox.CheckedCallback = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		barSettings.locked = self.barLockedCheckBox.IsChecked;
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end	
	self.barLockedCheckBox:SetChecked( localBarSettings.locked );
	self.sb.ValueChanged = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		barSettings.quickslotRows = self.sb:GetValue();
		barSettings.quickslotCount = barSettings.quickslotRows * barSettings.quickslotColumns;
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end	
	self.sb:SetValue( localBarSettings.quickslotRows );
	self.sb2.ValueChanged = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		barSettings.quickslotColumns = self.sb2:GetValue();
		barSettings.quickslotCount = barSettings.quickslotRows * barSettings.quickslotColumns;
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end	
	self.sb2:SetValue( localBarSettings.quickslotColumns );

	self.spacingSB.ValueChanged = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		barSettings.quickslotSpacing = self.spacingSB:GetValue();
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end
	self.spacingSB:SetValue( localBarSettings.quickslotSpacing );
	self.sizeSB.ValueChanged = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		barSettings.quickslotSize = self.sizeSB:GetValue() + 16;
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end
	self.sizeSB:SetValue( localBarSettings.quickslotSize - 16 );
	
    self.triggerList.SelectedIndexChanged = function(sender, args)
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		if ( self.triggerList:GetSelection() == 1 ) then
			barSettings.events.triggerOnClassBuffActive = true;
		else
			barSettings.events.triggerOnClassBuffActive = false;
		end
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end
	if ( localBarSettings.events.triggerOnClassBuffActive ) then
		self.triggerList:SetSelection( 1 );
	else
		self.triggerList:SetSelection( 2 );
	end
	
	--TRIGGERS
	self.combatBarCheckBox.CheckedCallback = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		if ( self.combatBarCheckBox.IsChecked ) then
			barSettings.events.displayInCombat = true;
		else
			barSettings.events.displayInCombat = nil;
		end
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end
	self.combatBarCheckBox:SetChecked( localBarSettings.events.displayInCombat );
	self.outOfCombatBarCheckBox.CheckedCallback = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		if ( self.outOfCombatBarCheckBox.IsChecked ) then
			barSettings.events.displayNotInCombat = true;
		else
			barSettings.events.displayNotInCombat = nil;
		end
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end
	self.outOfCombatBarCheckBox:SetChecked( localBarSettings.events.displayNotInCombat );
	self.altBarCheckBox.CheckedCallback = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		if ( self.altBarCheckBox.IsChecked ) then
			barSettings.events.isAlt = true;
		else
			barSettings.events.isAlt = nil;
		end
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end
	self.altBarCheckBox:SetChecked( localBarSettings.events.isAlt );
	self.controlBarCheckBox.CheckedCallback = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		if ( self.controlBarCheckBox.IsChecked ) then
			barSettings.events.isControl = true;
		else
			barSettings.events.isControl = nil;
		end
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end
	self.controlBarCheckBox:SetChecked( localBarSettings.events.isControl );
	self.shiftBarCheckBox.CheckedCallback = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		if ( self.shiftBarCheckBox.IsChecked ) then
			barSettings.events.isShift = true;
		else
			barSettings.events.isShift = nil;
		end
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end
	self.shiftBarCheckBox:SetChecked( localBarSettings.events.isShift );

	if ( self.eventCheckboxes ~= nil ) then
		for key, value in pairs (self.eventCheckboxes) do
			local checked = false;
			if ( localBarSettings.events.categories ~= nil and localBarSettings.events.categories[key] ~= nil ) then
				checked = true;
			end
			if ( localBarSettings.events.effects ~= nil and localBarSettings.events.effects[key] ~= nil ) then
				checked = true;
			end
			value:SetChecked( checked );
		end
	end

	self.healthTriggerCheckBox.CheckedCallback = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		if ( self.healthTriggerCheckBox.IsChecked ) then
			barSettings.events.displayOnHealth = true;
		else
			barSettings.events.displayOnHealth = nil;
		end
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end	
	self.healthTriggerCheckBox:SetChecked( localBarSettings.events.displayOnHealth );
	
	self.healthSB.ValueChanged = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		barSettings.events.healthTrigger = self.healthSB:GetValue() / 100;
		self:SetColors( barSettings );
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end
	self.healthSB:SetValue( localBarSettings.events.healthTrigger * 100 );

	self.powerTriggerCheckBox.CheckedCallback = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		if ( self.powerTriggerCheckBox.IsChecked ) then
			barSettings.events.displayOnPower = true;
		else
			barSettings.events.displayOnPower = nil;
		end
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end	
	self.powerTriggerCheckBox:SetChecked( localBarSettings.events.displayOnPower );

	self.powerSB.ValueChanged = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		barSettings.events.powerTrigger = self.powerSB:GetValue() / 100;
		self:SetColors( barSettings );
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end
	self.powerSB:SetValue( localBarSettings.events.powerTrigger * 100 );

	-- COLOR SETTINGS
	self.useBackgroundCheckBox.CheckedCallback = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		barSettings.useBackgroundColor = self.useBackgroundCheckBox.IsChecked;
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end	
	self.useBackgroundCheckBox:SetChecked( localBarSettings.useBackgroundColor );
	self.bgRSB.ValueChanged = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		barSettings.backgroundColorRed = self.bgRSB:GetValue() / 100;
		self:SetColors( barSettings );
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end
	self.bgRSB:SetValue( localBarSettings.backgroundColorRed * 100 );
	self.bgGSB.ValueChanged = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		barSettings.backgroundColorGreen = self.bgGSB:GetValue() / 100;
		self:SetColors( barSettings );
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end
	self.bgGSB:SetValue( localBarSettings.backgroundColorGreen * 100 );
	self.bgBSB.ValueChanged = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		barSettings.backgroundColorBlue = self.bgBSB:GetValue() / 100;
		self:SetColors( barSettings );
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end
	self.bgBSB:SetValue( localBarSettings.backgroundColorBlue * 100 );
	self.bgASB.ValueChanged = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		barSettings.opacity = self.bgASB:GetValue() / 100;
		self:SetColors( barSettings );
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end
	self.bgASB:SetValue( localBarSettings.opacity * 100 );

	
	self.fadeOpacitySelection.ValueChanged = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		barSettings.fadeOpacity = self.fadeOpacitySelection:GetValue() / 100;
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end
	self.fadeOpacitySelection:SetValue( localBarSettings.fadeOpacity * 100 );
	self.useFadingCheckBox.CheckedCallback = function( sender, args )
		local barSettings = theSettingsManager:GetBarSettings( self.barList:GetSelection() );
		barSettings.useFading = self.useFadingCheckBox.IsChecked;
		theSettingsManager:SetBarSettings( self.barList:GetSelection(), barSettings );
	end	
	self.useFadingCheckBox:SetChecked( localBarSettings.useFading );
end
function QuickslotsMenu:SetColors( barSettings )
	theColor = Turbine.UI.Color( barSettings.opacity, barSettings.backgroundColorRed, barSettings.backgroundColorGreen, barSettings.backgroundColorBlue )
end

function QuickslotsMenu:AddCommandsBar()
	self.qsCommandsBox = Tonic.UI.AutoListBox();
	self.qsCommandsBox:SetParent( self );
	self.qsCommandsBox:SetBackColor( unselectedColor );
	self.qsCommandsBox:SetOrientation( Turbine.UI.Orientation.Horizontal );
	self.qsCommandsBox:SetVisible( true );

	local addButton = Turbine.UI.Lotro.Button();
	addButton:SetText( LOCALESTRINGS.QuickslotsMenu["Add"] );
	addButton:SetSize( buttonWidth, selectionHeight );
	addButton.MouseClick = function( sender, args )
		local barid = maitre_d:Add( QUICKSLOTBAR );
		self:RefreshComboBox( barid );
	end
	self.qsCommandsBox:AddItem( addButton );

	local removeButton = Turbine.UI.Lotro.Button();
	removeButton:SetText( LOCALESTRINGS.QuickslotsMenu["Remove"] );
	removeButton:SetSize( buttonWidth, selectionHeight );

	removeButton.MouseClick = function( sender, args )
		maitre_d:Remove( tonumber( self.barList:GetSelection() ) );
		self.barList:RemoveItem( self.barList:GetSelection() );
	end
	self.qsCommandsBox:AddItem( removeButton );
	
	local copyButton = Turbine.UI.Lotro.Button();
	copyButton:SetText( LOCALESTRINGS.QuickslotsMenu["Copy"] );
	copyButton:SetSize( buttonWidth, selectionHeight );

	copyButton.MouseClick = function( sender, args )
		local barid = maitre_d:Copy( tonumber( self.barList:GetSelection() ) );
		self.barList:AddItem( barid, barid );
	end
	self.qsCommandsBox:AddItem( copyButton );
	
	local resetButton = Turbine.UI.Lotro.Button();
	resetButton:SetText( LOCALESTRINGS.QuickslotsMenu["Reset"] );
	resetButton:SetSize( buttonWidth + 10, selectionHeight );

	resetButton.MouseClick = function( sender, args )
		maitre_d:Reset( tonumber( self.barList:GetSelection() ) );
	end
	self.qsCommandsBox:AddItem( resetButton );
	
	self.upperBar:AddItem( self.qsCommandsBox );
end

function QuickslotsMenu:GetSelection()
	return self.barList:GetSelection();
end

function QuickslotsMenu:ClearSelection()
	local settings = theSettingsManager:GetSettings();
	settings.selectedBar = 0;
	self.settingsBox:SetVisible( false );
	self.barList:SetSelection();
end

function QuickslotsMenu:EnableTriggers( enabled )
	self.combatBarCheckBox:SetDisabled( not enabled );
	self.outOfCombatBarCheckBox:SetDisabled( not enabled );
	self.altBarCheckBox:SetDisabled( not enabled );
	self.controlBarCheckBox:SetDisabled( not enabled );
	self.shiftBarCheckBox:SetDisabled( not enabled );
	
	self.healthTriggerCheckBox:SetDisabled( not enabled );
	self.powerTriggerCheckBox:SetDisabled( not enabled );
	
	for key, value in pairs (self.eventCheckboxes) do
		value:SetDisabled( not enabled );
	end
end

function QuickslotsMenu:RefreshComboBox( barid )
	self.barList:Clear();
	for key, value in pairs (theSettingsManager:GetBars( QUICKSLOTBAR )) do
		local found = false;
		for i = 1, self.barList:GetItemCount() do
			if ( self.barList:GetItem(i) == key) then
				found = true;
			end
		end
		if ( found == false ) then
			local text = key;
			local barSettings = theSettingsManager:GetBarSettings( key );
			if ( barSettings.barName ~= nil ) then
				text = barSettings.barName;
			end
			self.barList:AddItem( text, key );
		end
	end
	if ( barid ~= nil ) then
		self.barList:SetSelection( barid );
	end
end